/**
 * @license
 * Copyright 2021 Google LLC. All Rights Reserved.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * =============================================================================
 */
import * as tflite from '@tensorflow/tfjs-tflite';
import { TaskModelLoader } from '../../task_model';
import { Runtime, Task } from '../common';
import { ImageClassifierTFLite } from './tflite_common';
declare type TFLiteNS = typeof tflite;
/** Loading options. */
export interface MobilenetTFLiteLoadingOptions extends tflite.ImageClassifierOptions {
    /**
     * The MobileNet version number. Use 1 for MobileNetV1, and 2 for
     * MobileNetV2. Defaults to 1.
     */
    version?: 1 | 2;
    /**
     * Controls the width of the network, trading accuracy for performance. A
     * smaller alpha decreases accuracy and increases performance. Defaults
     * to 1.0.
     */
    alpha?: 0.25 | 0.50 | 0.75 | 1.0;
}
/**
 * Inference options.
 *
 * TODO: placeholder for now.
 */
export interface MobilenetTFLiteInferenceOptions {
}
/** Loader for mobilenet TFLite model. */
export declare class MobilenetTFLiteLoader extends TaskModelLoader<TFLiteNS, MobilenetTFLiteLoadingOptions, MobilenetTFLite> {
    readonly metadata: {
        name: string;
        description: string;
        runtime: Runtime;
        version: string;
        supportedTasks: Task[];
    };
    readonly packageUrls: string[][];
    readonly sourceModelGlobalNamespace = "tflite";
    protected transformSourceModel(sourceModelGlobal: TFLiteNS, loadingOptions?: MobilenetTFLiteLoadingOptions): Promise<MobilenetTFLite>;
}
/**
 * Pre-trained TFLite mobilenet image classification model.
 *
 * Usage:
 *
 * ```js
 * // Load the model with options (optional).
 * //
 * // By default, it uses mobilenet V1. You can change it in the options
 * // parameter of the `load` function (see below for docs).
 * const model = await tfTask.ImageClassification.Mobilenet.TFJS.load();
 *
 * // Run inference on an image.
 * const img = document.querySelector('img');
 * const result = await model.predict(img);
 * console.log(result.classes);
 *
 * // Clean up.
 * model.cleanUp();
 * ```
 *
 * Refer to `tfTask.ImageClassifier` for the `predict` and `cleanUp` method.
 *
 * @docextratypes [
 *   {description: 'Options for `load`', symbol:
 * 'MobilenetTFLiteLoadingOptions'},
 *   {description: 'Options for `predict`',
 * symbol: 'MobilenetTFLiteInferenceOptions'}
 * ]
 *
 * @doc {heading: 'Image Classification', subheading: 'Models'}
 */
export declare class MobilenetTFLite extends ImageClassifierTFLite<MobilenetTFLiteInferenceOptions> {
}
export declare const mobilenetTfliteLoader: MobilenetTFLiteLoader;
export {};
