/**
 * @license
 * Copyright 2021 Google LLC. All Rights Reserved.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * =============================================================================
 */
import * as tflite from '@tensorflow/tfjs-tflite';
import { TaskModelLoader } from '../../task_model';
import { Runtime, Task } from '../common';
import { SentimentDetectionBaseOptions, SentimentDetectionResult, SentimentDetector } from './common';
declare type TFLiteNS = typeof tflite;
/** Loading options. */
export interface MovieReviewTFLiteLoadingOptions extends SentimentDetectionBaseOptions {
}
/**
 * Inference options.
 *
 * TODO: placeholder for now.
 */
export interface MovieReviewTFLiteInferenceOptions {
}
/** Loader for cocossd TFLite model. */
export declare class MovieReviewTFLiteLoader extends TaskModelLoader<TFLiteNS, MovieReviewTFLiteLoadingOptions, MovieReviewTFLite> {
    readonly metadata: {
        name: string;
        description: string;
        runtime: Runtime;
        version: string;
        supportedTasks: Task[];
    };
    readonly packageUrls: string[][];
    readonly sourceModelGlobalNamespace = "tflite";
    protected transformSourceModel(sourceModelGlobal: TFLiteNS, loadingOptions?: MovieReviewTFLiteLoadingOptions): Promise<MovieReviewTFLite>;
}
/**
 * Pre-trained TFLite movie review sentiment detection model.
 *
 * It detects whether the review text is positive or negetive.
 *
 * Usage:
 *
 * ```js
 * // Load the model with options (optional).
 * const model = await tfTask.SentimentDetection.MovieReview.TFLite.load();
 *
 * // Run inference on a review text.
 * const result = await model.predict('This is a great movie!');
 * console.log(result.sentimentLabels);
 *
 * // Clean up.
 * model.cleanUp();
 * ```
 *
 * The model returns the prediction results of the following sentiment labels:
 *
 * - positive
 * - negative
 *
 * Refer to `tfTask.SentimentDetector` for the `predict` and `cleanUp` method,
 * and more details about the result interface.
 *
 * @docextratypes [
 *   {description: 'Options for `load`', symbol:
 * 'MovieReviewTFLiteLoadingOptions'},
 *   {description: 'Options for `predict`',
 * symbol: 'MovieReviewTFLiteInferenceOptions'}
 * ]
 *
 * @doc {heading: 'Sentiment Detection', subheading: 'Models'}
 */
export declare class MovieReviewTFLite extends SentimentDetector<MovieReviewTFLiteInferenceOptions> {
    private tfliteNLClassifier;
    private threshold;
    constructor(tfliteNLClassifier: tflite.NLClassifier, threshold: number);
    predict(text: string, options?: MovieReviewTFLiteInferenceOptions): Promise<SentimentDetectionResult>;
}
export declare const movieReviewTfliteLoader: MovieReviewTFLiteLoader;
export {};
